@@ -205,6 +205,27 @@ PyObject *py_call_function(char *pkg, char *name, PyObject *args) {
     return py_retval;
 }
 
+PyObject *py_call_function_kw(char *pkg, char *name, PyObject *args, PyObject *kw) {
+    PyObject * const mod       = PyImport_AddModule(pkg);
+    PyObject * const dict      = PyModule_GetDict(mod);
+    PyObject * const func      = PyMapping_GetItemString(dict, name);
+    PyObject *py_retval = NULL;
+
+    if (func == NULL) {
+        PyErr_Format(PyExc_NameError, "name '%s' is not defined", name);
+        goto cleanup;
+    }
+
+    py_retval = PyObject_Call(func, args, kw);
+
+    Py_DECREF(func);
+    cleanup:
+    Py_DECREF(args);
+    Py_DECREF(kw);
+
+    return py_retval;
+}
+
 void py_fetch_error(PyObject **exception) {
     /* ex_type, ex_value, ex_trace, ex_message */
     PyErr_Fetch(&exception[0], &exception[1], &exception[2]);
@@ -228,6 +249,25 @@ void py_raise_missing_method(PyObject *obj, char *name) {
     }
 }
 
+PyObject *py_call_static_method_kw(char *pkg, char *class, char *name, PyObject *args, PyObject *kw) {
+    PyObject * const mod  = PyImport_AddModule(pkg);
+    PyObject * const dict = PyModule_GetDict(mod);
+    PyObject * const obj  = PyMapping_GetItemString(dict, class);
+    PyObject *method = PyObject_GetAttrString(obj, name);
+    if (method == NULL) {
+        py_raise_missing_method(obj, name);
+        goto cleanup;
+    }
+    PyObject *py_retval = PyObject_Call(method, args, kw);
+    Py_DECREF(method);
+
+    cleanup:
+    Py_DECREF(args);
+    Py_DECREF(kw);
+
+    return py_retval;
+}
+
 PyObject *py_call_static_method(char *pkg, char *class, char *name, PyObject *args) {
     PyObject * const mod  = PyImport_AddModule(pkg);
     PyObject * const dict = PyModule_GetDict(mod);
@@ -261,6 +301,21 @@ PyObject *py_call_method(PyObject *obj, char *name, PyObject *args) {
     return py_retval;
 }
 
+PyObject *py_call_method_kw(PyObject *obj, char *name, PyObject *args, PyObject *kw) {
+    PyObject *method = PyObject_GetAttrString(obj, name);
+    if (method == NULL) {
+        py_raise_missing_method(obj, name);
+        goto cleanup;
+    }
+    PyObject *py_retval = PyObject_Call(method, args, kw);
+    Py_DECREF(method);
+
+    cleanup:
+    Py_DECREF(args);
+
+    return py_retval;
+}
+
 static PyObject *perl6_call(PyObject *self, PyObject *args) {
     PyObject * const index  = PyTuple_GetItem(args, 0);
     PyObject * const params = PyTuple_GetItem(args, 1);
